import {Architecture, LoadedNewMap} from "./newmap.ts";
import {Grid, Point} from "./engine/datatypes.ts";
import {getThralls} from "./thralls.ts";
import {LadderPickup, ThrallPosterPickup} from "./pickups.ts";

const BASIC_PLAN = Grid.createGridFromMultilineString(`
#####################
#########   #########
##### A # L # D #####
##### a #   # d #####
#####   ## ##   #####
#   ## ##   ## ##   #
#Bb               eE#
#   ## ##   ## ##   #
#####   ## ##   #####
##### c #   # f #####
##### C # @ # F #####
#########   #########
#####################
`);

export function generateManor(): LoadedNewMap {
  let map = new LoadedNewMap("manor", BASIC_PLAN.size);
  let thralls = getThralls().getAll();

  for (let y = 0; y < BASIC_PLAN.size.h; y++) {
    for (let x = 0; x < BASIC_PLAN.size.w; x++) {
      let xy = new Point(x, y);
      let cell = map.get(xy);

      let placeThrall = (ix: number) => {
        // TODO
        cell.architecture = Architecture.Floor;
      };
      let placeThrallPoster = (ix: number) => {
        cell.architecture = Architecture.Floor;
        cell.pickup = new ThrallPosterPickup(thralls[ix]);
      };

      switch (BASIC_PLAN.get(xy)) {
        case '#': break
        case '@': cell.architecture = Architecture.Floor; map.entrance = xy; break;
        case 'L': cell.architecture = Architecture.Floor; cell.pickup = new LadderPickup(); break;
        case ' ': cell.architecture = Architecture.Floor; break;
        case 'a': placeThrall(0); break;
        case 'b': placeThrall(1); break;
        case 'c': placeThrall(2); break;
        case 'd': placeThrall(3); break;
        case 'e': placeThrall(4); break;
        case 'f': placeThrall(5); break;
        case 'A': placeThrallPoster(0); break;
        case 'B': placeThrallPoster(1); break;
        case 'C': placeThrallPoster(2); break;
        case 'D': placeThrallPoster(3); break;
        case 'E': placeThrallPoster(4); break;
        case 'F': placeThrallPoster(5); break;
      }
    }
  }

  return map;
}